// LabWindows/CVI Example using VISA_Extended library for easier communication with a VISA instrument
// This source file and the VISA_Extended library can be easiliy imported into a Visual Studio C++ project

#include <utility.h>
#include <ansi_c.h>
#include "VISA_Extended.h"

// error checking macro
#define CHECKERR(fCal) \
	status = (fCal);if (status < VI_SUCCESS) goto ERROR_HANDLING;

#define ARRAY_SIZES 200000

int main()
{
	ViSession rtb;
	ViStatus status = VI_SUCCESS;
	char response[VISAEXT_BUFFER_SIZE];
	char instrumentErrors[16384];
	char generalerror[16384];
	static double waveformASC[ARRAY_SIZES];
	static double waveformBIN[ARRAY_SIZES];
	int count;

	//-----------------------------------------------------------
	//Initialization:
	//-----------------------------------------------------------
    SetBreakOnLibraryErrors (0); // Don't show Pupup messages with errors from libraries. Remove, if used in VS C++ project
    CHECKERR(VISAext_Open("TCPIP::192.168.1.102::INSTR", &rtb));
	CHECKERR(VISAext_Query(rtb, "*IDN?", response));
	CHECKERR(VISAext_Write(rtb, "*RST;*CLS")); // Reset the instrument, clear the Error queue
	CHECKERR(VISAext_ErrorChecking(rtb, instrumentErrors)); // Error Checking after Initialization block
	//-----------------------------------------------------------
	// Basic Settings:
	//---------------------------- -------------------------------
	CHECKERR(VISAext_Write(rtb, "TIM:ACQT 0.01")); //10ms Acquisition time
	CHECKERR(VISAext_Write(rtb, "CHAN1:RANG 5.0")); // Horizontal range 5V (0.5V/div)
	CHECKERR(VISAext_Write(rtb, "CHAN1:OFFS 0.0")); // Offset 0
	CHECKERR(VISAext_Write(rtb, "CHAN1:COUP ACL")); // Coupling AC 1MOhm
	CHECKERR(VISAext_Write(rtb, "CHAN1:STAT ON")); // Switch Channel 1 ON
	CHECKERR(VISAext_ErrorChecking(rtb, instrumentErrors)); // Error Checking after Basic Settings block
	//-----------------------------------------------------------
	// Trigger Settings:
	//-----------------------------------------------------------
	CHECKERR(VISAext_Write(rtb, "TRIG:A:MODE AUTO")); // Trigger Auto mode in case of no signal is applied
	CHECKERR(VISAext_Write(rtb, "TRIG:A:TYPE EDGE;:TRIG:A:EDGE:SLOP POS")); // Trigger type Edge Positive
	CHECKERR(VISAext_Write(rtb, "TRIG:A:SOUR CH1")); // Trigger source CH1
	CHECKERR(VISAext_Write(rtb, "TRIG:A:LEV1 0.05")); // Trigger level 0.05V
	CHECKERR(VISAext_Query(rtb, "*OPC?", response)); // Using *OPC? query waits until all the instrument settings are finished
	CHECKERR(VISAext_ErrorChecking(rtb, instrumentErrors)); // Error Checking after Trigger Settings block
	// -----------------------------------------------------------
	// SyncPoint 'SettingsApplied' - all the settings were applied
	// -----------------------------------------------------------
	// Arming the SCOPE for single acquisition
	// -----------------------------------------------------------
    CHECKERR(VISAext_SetTimeout(rtb, 2000)); // Acquisition timeout - set it higher than the acquisition time
    CHECKERR(VISAext_Write(rtb, "SING"));
	// -----------------------------------------------------------
	// DUT_Generate_Signal() - in our case we use Probe compensation signal
	// where the trigger event (positive edge) is reoccuring
	// -----------------------------------------------------------
	CHECKERR(VISAext_Query(rtb, "*OPC?", response)); // Using *OPC? query waits until the instrument finished the Acquisition
	CHECKERR(VISAext_ErrorChecking(rtb, instrumentErrors)); // Error Checking after the acquisition is finished
	// -----------------------------------------------------------
	// SyncPoint 'AcquisitionFinished' - the results are ready
	// -----------------------------------------------------------
	// Fetching the waveform in ASCII format
	// -----------------------------------------------------------
	printf("Fetching waveform in ASCII format... ");
	CHECKERR(VISAext_Write(rtb, "FORM ASC;:CHAN1:DATA?"));
	CHECKERR(VISAext_ReadListOfDoubles(rtb, waveformASC, ARRAY_SIZES, &count));
	CHECKERR(VISAext_ErrorChecking(rtb, instrumentErrors)); // Error Checking after the data transfer
	printf("Samples count: %d\n", count);
	// -----------------------------------------------------------
	// Fetching the trace in Binary format
	// Transfer of traces in binary format is faster.
	// The waveformBIN data and waveformASC data are however the same.
	// -----------------------------------------------------------
	printf("Fetching waveform in Binary format... ");
	CHECKERR(VISAext_Write(rtb, "FORM REAL;:CHAN1:DATA?"));
	CHECKERR(VISAext_ReadBinaryFloatData(rtb, 1, waveformBIN, ARRAY_SIZES, &count));
	CHECKERR(VISAext_ErrorChecking(rtb, instrumentErrors)); // Error Checking after the data transfer
	printf("Samples count: %d\n", count);
	// -----------------------------------------------------------
	// Making an instrument screenshot and transferring the file to the PC
	// -----------------------------------------------------------
	CHECKERR(VISAext_Write(rtb, "MMEM:DEL 'Dev_Screenshot.png'")); // Delete the file if it already exists
	CHECKERR(VISAext_ClearStatus(rtb)); // if the 'Dev_Screenshot.png' didn't exist, the instrument generates an error. Clear it.
	CHECKERR(VISAext_Write(rtb, "HCOP:LANG PNG;:MMEM:NAME 'Dev_Screenshot'")); // Hardcopy settings for taking a screenshot - notice no file extention here
	CHECKERR(VISAext_Write(rtb, "HCOP:IMM")); // Make the screenshot now
	CHECKERR(VISAext_Query(rtb, "*OPC?", response)); // Wait for the screenshot to be saved
	CHECKERR(VISAext_ErrorChecking(rtb, instrumentErrors)); // Error Checking after the screenshot creation
	CHECKERR(VISAext_Write(rtb, "MMEM:DATA? 'Dev_Screenshot.png'")); // Query the instrument file
	CHECKERR(VISAext_ReadBinaryDataToFile(rtb, "c:\\Temp\\PC_Screenshot.png")); // Read the response and store the file to the PC
	CHECKERR(VISAext_ErrorChecking(rtb, instrumentErrors)); // Error Checking at the end of the program
	printf("\nScreenshot file saved to PC 'c:\\Temp\\PC_Screenshot.png'");

ERROR_HANDLING:
	if (status == VI_SUCCESS)
	{
		; // all went OK
	}
	else if (status == VISAEXT_ERR_INSTRUMENT_STATUS) 
	{
		//Instrument error occured
		printf("\nThe following instrument error(s) occured:\n%s", instrumentErrors);
	}
	else
	{
		//general error occured, try to get the string description from the status code
		VISAext_ErrorCodeDescription(rtb, status, generalerror);
		printf("\nThe following error(s) occured:\n%s", generalerror);
	}

	printf("\n\nPress ENTER to continue...");
	(void)getchar();
	
	VISAext_Close(rtb);
	
	return 0;
}
