// LabWindows/CVI Example using VISA_Extended library for easier communication with a VISA instrument
// This source file and the VISA_Extended library can be easiliy imported into a Visual Studio C++ project

#include <utility.h>
#include <ansi_c.h>
#include "VISA_Extended.h"

// error checking macro
#define CHECKERR(fCal) \
	status = (fCal);if (status < VI_SUCCESS) goto ERROR_HANDLING;

#define ARRAY_SIZES 200000

int main()
{
	ViSession specan;
	ViStatus status = VI_SUCCESS;
	char response[VISAEXT_BUFFER_SIZE];
	char instrumentErrors[16384];
	char generalerror[16384];
	static double traceASC[ARRAY_SIZES];
	static double traceBIN[ARRAY_SIZES];
	double markerX;
	double markerY;
	int count;

	//-----------------------------------------------------------
	// Initialization:
	//-----------------------------------------------------------
	SetBreakOnLibraryErrors (0); // Don't show Pupup messages with errors from libraries. Remove, if used in VS C++ project
    CHECKERR(VISAext_Open("TCPIP::localhost::INSTR", &specan));
	CHECKERR(VISAext_Query(specan, "*IDN?", response));
	CHECKERR(VISAext_Write(specan, "*RST;*CLS")); // Reset the instrument, clear the Error queue
	CHECKERR(VISAext_Write(specan, "INIT:CONT OFF")); // Switch OFF the continuous sweep
	CHECKERR(VISAext_Write(specan, "SYST:DISP:UPD ON")); // Display update ON - switch OFF after debugging
	CHECKERR(VISAext_ErrorChecking(specan, instrumentErrors)); // Error Checking after Initialization block
	//-----------------------------------------------------------
	// Basic Settings:
	//-----------------------------------------------------------
	CHECKERR(VISAext_Write(specan, "DISP:WIND:TRAC:Y:RLEV 10.0")); // Setting the Reference Level
	CHECKERR(VISAext_Write(specan, "FREQ:CENT 3.0 GHz")); // Setting the center frequency
	CHECKERR(VISAext_Write(specan, "FREQ:SPAN 200 MHz")); // Setting the span
	CHECKERR(VISAext_Write(specan, "BAND 100 kHz")); // Setting the RBW
	CHECKERR(VISAext_Write(specan, "BAND:VID 300kHz")); // Setting the VBW
    CHECKERR(VISAext_Write(specan, "SWE:POIN 10001")); // Setting the sweep points
	CHECKERR(VISAext_Query(specan, "*OPC?", response)); //Using *OPC? query waits until all the instrument settings are finished
	CHECKERR(VISAext_ErrorChecking(specan, instrumentErrors)); // Error Checking after Basic Settings block
	// -----------------------------------------------------------
	// SyncPoint 'SettingsApplied' - all the settings were applied
	// -----------------------------------------------------------
    CHECKERR(VISAext_SetTimeout(specan, 2000)); // Sweep timeout - set it higher than the instrument measurement time
    CHECKERR(VISAext_Write(specan, "INIT")); // Start the sweep
	CHECKERR(VISAext_Query(specan, "*OPC?", response)); // Using *OPC? query waits until the instrument finished the Acquisition
	CHECKERR(VISAext_ErrorChecking(specan, instrumentErrors)); // Error Checking after the acquisition is finished
	// -----------------------------------------------------------
	// SyncPoint 'AcquisitionFinished' - the results are ready
	// -----------------------------------------------------------
	// Fetching the trace in ASCII format
	// -----------------------------------------------------------
	printf("Fetching trace in ASCII format... ");
	CHECKERR(VISAext_Write(specan, "FORM ASC;:TRAC? TRACE1"));
	CHECKERR(VISAext_ReadListOfDoubles(specan, traceASC, ARRAY_SIZES, &count));
	printf("Samples count: %d\n", count);
	CHECKERR(VISAext_ErrorChecking(specan, instrumentErrors)); // Error Checking after the data transfer
	// -----------------------------------------------------------
	// Fetching the trace in Binary format
	// The transfer time of traces in binary format is shorter.
	// The traceBIN data and traceASC data are however the same.
	// -----------------------------------------------------------
	printf("Fetching trace in Binary format... ");
	CHECKERR(VISAext_Write(specan, "FORM REAL,32;:TRAC? TRACE1"))
	CHECKERR(VISAext_ReadBinaryFloatData(specan, 1, traceBIN, ARRAY_SIZES, &count))
	printf("Samples count: %d\n", count);
	CHECKERR(VISAext_ErrorChecking(specan, instrumentErrors)); // Error Checking after the data transfer
	// -----------------------------------------------------------
	// Setting the marker to max and querying the X and Y
	// -----------------------------------------------------------
	CHECKERR(VISAext_Write(specan, "CALC1:MARK1:MAX")); // Set the marker to the maximum point of the entire trace
	CHECKERR(VISAext_Query(specan, "*OPC?", response)); // Using *OPC? query waits until the marker is set
	CHECKERR(VISAext_QueryDouble(specan, "CALC1:MARK1:X?", &markerX));
	CHECKERR(VISAext_QueryDouble(specan, "CALC1:MARK1:Y?", &markerY));
    CHECKERR(VISAext_ErrorChecking(specan, instrumentErrors)); // Error Checking after the markers reading   
	printf("Marker Frequency %0.3f MHz, Level %0.2f dBm", markerX/1E6, markerY);
	// -----------------------------------------------------------
	// Making an instrument screenshot and transferring the file to the PC
	// -----------------------------------------------------------
	CHECKERR(VISAext_Write(specan, "HCOP:DEV:LANG PNG"));       
	CHECKERR(VISAext_Write(specan, "MMEM:NAME 'c:\\temp\\Dev_Screenshot.png'"));
	CHECKERR(VISAext_Write(specan, "HCOP:IMM")); // Make the screenshot now
	CHECKERR(VISAext_Query(specan, "*OPC?", response)); // Wait for the screenshot to be saved
	CHECKERR(VISAext_ErrorChecking(specan, instrumentErrors)); // Error Checking after the screenshot creation
	CHECKERR(VISAext_Write(specan, "MMEM:DATA? 'c:\\temp\\Dev_Screenshot.png'"));
	CHECKERR(VISAext_ReadBinaryDataToFile(specan, "c:\\Temp\\PC_Screenshot.png")); // Read the response and store to the file in PC
	CHECKERR(VISAext_ErrorChecking(specan, instrumentErrors)); // Error Checking at the end of the program
	printf("\nScreenshot file saved to PC 'c:\\Temp\\PC_Screenshot.png'");

ERROR_HANDLING:
	if (status == VI_SUCCESS)
	{
		; // all went OK
	}
	else if (status == VISAEXT_ERR_INSTRUMENT_STATUS)
	{
		//Instrument error occured
		printf("\nThe following instrument error(s) occured:\n%s", instrumentErrors);
	}
	else
	{
		//general error occured, try to get the string description from the status code
		VISAext_ErrorCodeDescription(specan, status, generalerror);
		printf("\nThe following error(s) occured:\n%s", generalerror);
	}

	printf("\n\nPress ENTER to continue...");
	(void)getchar();

	VISAext_Close(specan);

	return 0;
}
